#/usr/bin/env julia
ENV["MPLBACKEND"] = "Agg"
import Granular
import PyPlot
import ArgParse
using Random
import DelimitedFiles

let
render = false

function parse_command_line()
    s = ArgParse.ArgParseSettings()
    ArgParse.@add_arg_table s begin
        "--E"
            help = "Young's modulus [Pa]"
            arg_type = Float64
            default = 2e7
        "--nu"
            help = "Poisson's ratio [-]"
            arg_type = Float64
            default = 0.285
        "--mu_d"
            help = "dynamic friction coefficient [-]"
            arg_type = Float64
            default = 0.3
        "--tensile_strength"
            help = "the maximum tensile strength [Pa]"
            arg_type = Float64
            default = 400e3
        "--tensile_strength_std_dev"
            help = "standard deviation of the maximum tensile strength [Pa]"
            arg_type = Float64
            default = 0.0
        "--shear_strength"
            help = "the maximum shear strength [Pa]"
            arg_type = Float64
            default = 200e3
        "--fracture_toughness"
            help = "fracture toughness [Pa*m^{1/2}]"
            arg_type = Float64
            default = 1e20
            #default = 1285e3
        "--r_min"
            help = "min. grain radius [m]"
            arg_type = Float64
            default = 20.0
        "--r_max"
            help = "max. grain radius [m]"
            arg_type = Float64
            default = 80.0
        "--thickness"
            help = "grain thickness [m]"
            arg_type = Float64
            default = 1.
        "--rotating"
            help = "allow the grains to rotate"
            arg_type = Bool
            default = true
        "--compressive_velocity"
            help = "compressive velocity [m/s]"
            arg_type = Float64
            default = 0.1
        "--seed"
            help = "seed for the pseudo RNG"
            arg_type = Int
            default = 1
        "id"
            help = "simulation id"
            required = true
    end
    return ArgParse.parse_args(s)
end

function report_args(parsed_args)
    println("Parsed args:")
    for (arg,val) in parsed_args
        println("  $arg  =>  $val")
    end
end

function run_simulation(id::String,
                        E::Float64,
                        nu::Float64,
                        mu_d::Float64,
                        tensile_strength::Float64,
                        tensile_strength_std_dev::Float64,
                        shear_strength::Float64,
                        fracture_toughness::Float64,
                        r_min::Float64,
                        r_max::Float64,
                        thickness::Float64,
                        rotating::Bool,
                        compressive_velocity::Float64,
                        seed::Int)

    ############################################################################
    #### SIMULATION PARAMETERS
    ############################################################################

    # Common simulation identifier
    id_prefix = id

    # Grain mechanical properties
    youngs_modulus = E              # Elastic modulus [Pa]
    poissons_ratio = nu             # Shear-stiffness ratio [-]
    contact_dynamic_friction = mu_d # Coulomb-frictional coefficient [-]

    # Simulation duration [s]
    t_total = 60.0 * 60.0 * 4.0

    # Temporal interval between output files
    file_dt = 18.0

    # Misc parameters
    water_density = 1000.

    # World size [m]
    L = [5e3, 2e4, 10.0]

    Random.seed!(seed)

    ############################################################################
    #### Create ice floes
    ############################################################################
    sim = Granular.createSimulation("$(id_prefix)")
    Granular.removeSimulationFiles(sim)

    # Create box edges of ice floes with r_min radius, moving inward
    r_walls = r_min

    # Create a grid for contact searching spanning the extent of the grains
    n = [Int(ceil(L[1]/r_max/2)), Int(ceil(L[2]/r_max/2)), 2]
    sim.ocean = Granular.createRegularOceanGrid(n, L)
    Granular.setGridBoundaryConditions!(sim.ocean, "impermeable", "-x +x")
    Granular.setGridBoundaryConditions!(sim.ocean, "impermeable", "-y +y")

    # Add unconstrained ice floes
    Granular.irregularPacking!(sim,
                               radius_max=r_max, radius_min=r_min,
                               thickness=thickness,
                               #binary_radius_search=true,
                               seed=seed)

    # Set grain mechanical properties
    for grain in sim.grains
        grain.youngs_modulus = youngs_modulus
        grain.poissons_ratio = poissons_ratio
        grain.tensile_strength = abs(tensile_strength +
                                     randn()*tensile_strength_std_dev)
        grain.shear_strength = abs(shear_strength +
                                   randn()*tensile_strength_std_dev)
        grain.contact_static_friction = contact_dynamic_friction  # mu_s = mu_d
        grain.contact_dynamic_friction = contact_dynamic_friction
        grain.fracture_toughness = fracture_toughness
        grain.rotating = rotating
        grain.ocean_drag_coeff_horiz = 0.0 # don't include drag on ends
    end

    # Create GSD plot to signify that simulation is complete
    Granular.writeSimulation(sim)
    render && Granular.plotGrainSizeDistribution(sim)

	# Add a dynamic wall to the top which adds a normal stress downwards.
	# The normal of this wall is downwards, and we place it at the top of
	# the granular assemblage.  Here, the fluid drag is also disabled
	y_top = -Inf
	for grain in sim.grains
		if y_top < grain.lin_pos[2] + grain.contact_radius
			y_top = grain.lin_pos[2] + grain.contact_radius
		end
	end
	Granular.addWallLinearFrictionless!(sim, [0., 1.], y_top,
										bc="normal stress",
										normal_stress=-20e3)
	sim.walls[1].mass *= 0.1  # set wall mass to 10% of total grain mass
	@info("Placing top wall at y=$y_top")

	# Resize the grid to span the current state
	Granular.fitGridToGrains!(sim, sim.ocean)

    Granular.setTimeStep!(sim)
    Granular.setTotalTime!(sim, t_total)
    Granular.setOutputFileInterval!(sim, file_dt)
    render && Granular.plotGrains(sim, verbose=true)

    ############################################################################
    #### RUN THE SIMULATION
    ############################################################################

	Granular.run!(sim)
	Granular.writeSimulation(sim)

    render && Granular.render(sim, trim=false, animation=true)
end

function main()
    parsed_args = parse_command_line()
    report_args(parsed_args)

    seed = parsed_args["seed"]
    run_simulation(parsed_args["id"] * "-seed" * string(seed),
                   parsed_args["E"],
                   parsed_args["nu"],
                   parsed_args["mu_d"],
                   parsed_args["tensile_strength"],
                   parsed_args["tensile_strength_std_dev"],
                   parsed_args["shear_strength"],
                   parsed_args["fracture_toughness"],
                   parsed_args["r_min"],
                   parsed_args["r_max"],
                   parsed_args["thickness"],
                   parsed_args["rotating"],
                   parsed_args["compressive_velocity"],
                   seed)
end

main()
end
